==> devchat-0.1.8.tar.gz: devchat-0.1.8/devchat/_cli.py <==
"""
This module contains the main function for the DevChat CLI.
"""
from contextlib import contextmanager
import json
import os
import sys
from typing import List, Optional, Tuple
import rich_click as click
from devchat.store import Store
from devchat.openai import OpenAIChatConfig, OpenAIChat
from devchat.assistant import Assistant
from devchat.utils import find_git_root, git_ignore, parse_files, setup_logger


click.rich_click.USE_MARKDOWN = True

logger = setup_logger(__name__)


@click.group()
def main():
    """DevChat CLI: A command-line interface for the DevChat chatbot."""


@contextmanager
def handle_errors():
    """Handle errors in the CLI."""
    try:
        yield
    except Exception as error:
        logger.exception(error)
        click.echo(f"Error: {error}")
        sys.exit(os.EX_SOFTWARE)


def init_dir() -> Tuple[dict, str]:
    git_root = find_git_root()
    chat_dir = os.path.join(git_root, ".chat")
    if not os.path.exists(chat_dir):
        os.makedirs(chat_dir)

    default_config_data = {
        'model': 'gpt-4',
        'provider': 'OpenAI',
        'OpenAI': {
            'temperature': 0,
            'stream': True
        }
    }

    try:
        with open(os.path.join(chat_dir, 'config.json'), 'r', encoding='utf-8') as file:
            config_data = json.load(file)
    except FileNotFoundError:
        config_data = default_config_data

    git_ignore(git_root, chat_dir)
    return config_data, chat_dir


@main.command()
@click.argument('content', required=False)
@click.option('-p', '--parent', help='Input the parent prompt hash to continue the conversation.')
@click.option('-r', '--reference', multiple=True,
              help='Input one or more specific previous prompts to include in the current prompt.')
@click.option('-i', '--instruct', multiple=True,
              help='Add one or more files to the prompt as instructions.')
@click.option('-c', '--context', multiple=True,
              help='Add one or more files to the prompt as a context.')
@click.option('-m', '--model', help='Specify the model to temporarily use for the prompt '
              '(prefer to modify .chat/config.json).')
def prompt(content: Optional[str], parent: Optional[str], reference: Optional[List[str]],
           instruct: Optional[List[str]], context: Optional[List[str]], model: Optional[str]):
    """
    Main function to run the chat application.

    This function initializes the chat system based on the specified large language model (LLM),
    and performs interactions with the LLM by sending prompts and retrieving responses.

    Examples
    --------

    To send a single-line message to the LLM, provide the content as an argument:

    ```bash
    devchat prompt "What is the capital of France?"
    ```

    To send a multi-line message to the LLM, use the here-doc syntax:

    ```bash
    devchat prompt << 'EOF'
    What is the capital of France?
    Can you tell me more about its history?
    EOF
    ```

    Note the quotes around EOF in the first line, to prevent the shell from expanding variables.

    Configuration
    -------------

    DevChat CLI reads its configuration from `.chat/config.json` in your current Git root directory.
    If the file is not found, it uses the following default configuration:
    ```json
    {
        "model": "gpt-3.5-turbo",
        "tokens-per-prompt": 3000,
        "provider": "OpenAI",
        "OpenAI": {
            "temperature": 0
        }
    }
    ```

    To customize the configuration, create `.chat/config.json` in your current Git root directory
    and modify the settings as needed. We recoommend the following settings:
    ```json
    {
        "model": "gpt-4",
        "tokens-per-prompt": 6000,
        "provider": "OpenAI",
        "OpenAI": {
            "temperature": 0,
            "stream": true
        }
    }
    ```

    Note: To use OpenAI's APIs, you must have an API key to run the CLI.
    Run the following command line with your API key:

    ```bash
    export OPENAI_API_KEY="sk-..."
    ```

    """
    config, chat_dir = init_dir()

    with handle_errors():
        if content is None:
            content = click.get_text_stream('stdin').read()

        if content == '':
            return

        instruct_contents = parse_files(instruct)
        context_contents = parse_files(context)

        provider = config.get('provider')
        if provider == 'OpenAI':
            if model is None:
                model = config['model']
            openai_config = OpenAIChatConfig(model=model, **config['OpenAI'])

            chat = OpenAIChat(openai_config)
            store = Store(chat_dir, chat)

            assistant = Assistant(chat, store)
            if 'tokens-per-prompt' in config:
                assistant.token_limit = config['tokens-per-prompt']

            assistant.make_prompt(content, instruct_contents, context_contents,
                                  parent, reference)

            for response in assistant.iterate_response():
                click.echo(response, nl=False)
        else:
            click.echo(f"Error: Invalid LLM in configuration '{provider}'")
            sys.exit(os.EX_DATAERR)


@main.command()
@click.option('--skip', default=0, help='Skip number prompts before showing the prompt history.')
@click.option('--max-count', default=100, help='Limit the number of commits to output.')
def log(skip, max_count):
    """
    Show the prompt history.
    """
    config, chat_dir = init_dir()
    provider = config.get('provider')
    recent_prompts = []
    if provider == 'OpenAI':
        openai_config = OpenAIChatConfig(model=config['model'], **config['OpenAI'])
        chat = OpenAIChat(openai_config)
        store = Store(chat_dir, chat)
        recent_prompts = store.select_recent(skip, skip + max_count)
    else:
        click.echo(f"Error: Invalid LLM in configuration '{provider}'")
        sys.exit(os.EX_DATAERR)

    logs = []
    for record in recent_prompts:
        try:
            logs.append(record.shortlog())
        except Exception:
            continue
    click.echo(json.dumps(logs, indent=2))

==> devchat-0.1.9.tar.gz: devchat-0.1.9/devchat/_cli.py <==
"""
This module contains the main function for the DevChat CLI.
"""
from contextlib import contextmanager
import json
import logging
import os
import sys
from typing import List, Optional, Tuple
import rich_click as click
from devchat.store import Store
from devchat.openai import OpenAIChatConfig, OpenAIChat
from devchat.assistant import Assistant
from devchat.utils import find_git_root, git_ignore, parse_files, setup_logger


click.rich_click.USE_MARKDOWN = True


@click.group()
def main():
    """DevChat CLI: A command-line interface for the DevChat chatbot."""


@contextmanager
def handle_errors():
    """Handle errors in the CLI."""
    try:
        yield
    except Exception as error:
        logger = logging.getLogger(__name__)
        logger.exception(error)
        click.echo(f"Error: {error}", err=True)
        sys.exit(os.EX_SOFTWARE)


def init_dir() -> Tuple[dict, str]:
    git_root = find_git_root()
    chat_dir = os.path.join(git_root, ".chat")
    if not os.path.exists(chat_dir):
        os.makedirs(chat_dir)

    default_config_data = {
        'model': 'gpt-4',
        'provider': 'OpenAI',
        'OpenAI': {
            'temperature': 0,
            'stream': True
        }
    }

    try:
        with open(os.path.join(chat_dir, 'config.json'), 'r', encoding='utf-8') as file:
            config_data = json.load(file)
    except FileNotFoundError:
        config_data = default_config_data

    setup_logger(os.path.join(chat_dir, 'error.log'))
    git_ignore(chat_dir, '*')
    return config_data, chat_dir


@main.command()
@click.argument('content', required=False)
@click.option('-p', '--parent', help='Input the parent prompt hash to continue the conversation.')
@click.option('-r', '--reference', multiple=True,
              help='Input one or more specific previous prompts to include in the current prompt.')
@click.option('-i', '--instruct', multiple=True,
              help='Add one or more files to the prompt as instructions.')
@click.option('-c', '--context', multiple=True,
              help='Add one or more files to the prompt as a context.')
@click.option('-m', '--model', help='Specify the model to temporarily use for the prompt '
              '(prefer to modify .chat/config.json).')
def prompt(content: Optional[str], parent: Optional[str], reference: Optional[List[str]],
           instruct: Optional[List[str]], context: Optional[List[str]], model: Optional[str]):
    """
    Main function to run the chat application.

    This function initializes the chat system based on the specified large language model (LLM),
    and performs interactions with the LLM by sending prompts and retrieving responses.

    Examples
    --------

    To send a single-line message to the LLM, provide the content as an argument:

    ```bash
    devchat prompt "What is the capital of France?"
    ```

    To send a multi-line message to the LLM, use the here-doc syntax:

    ```bash
    devchat prompt << 'EOF'
    What is the capital of France?
    Can you tell me more about its history?
    EOF
    ```

    Note the quotes around EOF in the first line, to prevent the shell from expanding variables.

    Configuration
    -------------

    DevChat CLI reads its configuration from `.chat/config.json` in your current Git root directory.
    If the file is not found, it uses the following default configuration:
    ```json
    {
        "model": "gpt-3.5-turbo",
        "tokens-per-prompt": 3000,
        "provider": "OpenAI",
        "OpenAI": {
            "temperature": 0
        }
    }
    ```

    To customize the configuration, create `.chat/config.json` in your current Git root directory
    and modify the settings as needed. We recoommend the following settings:
    ```json
    {
        "model": "gpt-4",
        "tokens-per-prompt": 6000,
        "provider": "OpenAI",
        "OpenAI": {
            "temperature": 0,
            "stream": true
        }
    }
    ```

    Note: To use OpenAI's APIs, you must have an API key to run the CLI.
    Run the following command line with your API key:

    ```bash
    export OPENAI_API_KEY="sk-..."
    ```

    """
    config, chat_dir = init_dir()

    with handle_errors():
        if content is None:
            content = click.get_text_stream('stdin').read()

        if content == '':
            return

        instruct_contents = parse_files(instruct)
        context_contents = parse_files(context)

        provider = config.get('provider')
        if provider == 'OpenAI':
            if model is None:
                model = config['model']
            openai_config = OpenAIChatConfig(model=model, **config['OpenAI'])

            chat = OpenAIChat(openai_config)
            store = Store(chat_dir, chat)

            assistant = Assistant(chat, store)
            if 'tokens-per-prompt' in config:
                assistant.token_limit = config['tokens-per-prompt']

            assistant.make_prompt(content, instruct_contents, context_contents,
                                  parent, reference)

            for response in assistant.iterate_response():
                click.echo(response, nl=False)
        else:
            click.echo(f"Error: Invalid LLM in configuration '{provider}'", err=True)
            sys.exit(os.EX_DATAERR)


@main.command()
@click.option('--skip', default=0, help='Skip number prompts before showing the prompt history.')
@click.option('--max-count', default=100, help='Limit the number of commits to output.')
def log(skip, max_count):
    """
    Show the prompt history.
    """
    config, chat_dir = init_dir()
    provider = config.get('provider')
    recent_prompts = []
    if provider == 'OpenAI':
        openai_config = OpenAIChatConfig(model=config['model'], **config['OpenAI'])
        chat = OpenAIChat(openai_config)
        store = Store(chat_dir, chat)
        recent_prompts = store.select_recent(skip, skip + max_count)
    else:
        click.echo(f"Error: Invalid LLM in configuration '{provider}'", err=True)
        sys.exit(os.EX_DATAERR)

    logs = []
    for record in recent_prompts:
        try:
            logs.append(record.shortlog())
        except Exception:
            continue
    click.echo(json.dumps(logs, indent=2))
